@@ -54,17 +54,12 @@ checkWrapperArgs = function(id, type, mart){
 }
 
 
-bmRequest <- function(request, ssl.verifypeer = TRUE, verbose = FALSE){
+bmRequest <- function(request, verbose = FALSE){
     if(verbose) writeLines(paste("Attempting web service request:\n",request, sep=""))
-    #result = tryCatch(getURL(request, ssl.verifypeer = ssl.verifypeer, followlocation = TRUE), 
-    #                  error = function(e){ 
-    #                      message("Request to BioMart web service failed.\n", 
-    #                              "The BioMart web service you're accessing may be down.\n",
-    #                              "Check the following URL and see if this website is available:\n",
-    #                              request)
-    #                  })
-    result <- httr::GET(request, content_type("text/plain"),
-                        set_cookies(.cookies = c(redirect_mirror = 'no')))
+
+    result <- httr::GET(request, content_type("text/plain")
+                        #, set_cookies(.cookies = c(redirect_mirror = 'no'))
+                        )
     stop_for_status(result)
     result2 <- content(result, encoding = "UTF-8")
     if(is.na(result2)) {
@@ -80,22 +75,35 @@ bmRequest <- function(request, ssl.verifypeer = TRUE, verbose = FALSE){
 #BioMart databases are present                        #
 #######################################################
 
-listMarts <- function( mart = NULL, host="www.ensembl.org", path="/biomart/martservice", port=80, includeHosts = FALSE, archive = FALSE, ssl.verifypeer = TRUE, ensemblRedirect = NULL, verbose = FALSE){
+listMarts <- function( mart = NULL, host="www.ensembl.org", path="/biomart/martservice", 
+                       port, includeHosts = FALSE, archive = FALSE, ensemblRedirect = NULL, verbose = FALSE){
     
-    if(!is.null(ensemblRedirect)) {
-        warning('The argument "ensemblRedirect" has been deprecated and will be removed in the next biomaRt release.')
-        ensemblRedirect <- NULL
+    if(missing(port)) {
+        port <- ifelse(grepl("https", host), yes = 443, no = 80)
     }
     
+    if(!is.null(ensemblRedirect)) {
+        warning('The argument "ensemblRedirect" has been deprecated and does not do anything.',
+                '\nSee ?useEnsembl for details on using mirror sites.')
+    }
+    
+    .listMarts(mart = mart, host = host, path = path, port = port, includeHosts = includeHosts,
+                archive = archive, verbose = verbose, ensemblRedirect = TRUE)
+    
+}
+
+.listMarts <- function( mart = NULL, host="www.ensembl.org", path="/biomart/martservice", 
+                       port=80, includeHosts = FALSE, archive = FALSE, verbose = FALSE, 
+                       ensemblRedirect = NULL){
+
     request = NULL
     if(is.null(mart)){
         
         host <- .cleanHostURL(host)
         if(archive) {
             stop("The archive = TRUE argument is now defunct.\nUse listEnsemblArchives() to find the URL to directly query an Ensembl archive.")
-        } 
-        else {
-            request = paste0(host, ":", port, path, "?type=registry&requestid=biomaRt")
+        } else {
+            request <- paste0(host, ":", port, path, "?type=registry&requestid=biomaRt")
         }
     }
     else{
@@ -107,7 +115,10 @@ listMarts <- function( mart = NULL, host="www.ensembl.org", path="/biomart/marts
         }
     } 	
     
-    registry = bmRequest(request = request, ssl.verifypeer = ssl.verifypeer, verbose = verbose)
+    if(!ensemblRedirect && grepl(x = request, pattern = "ensembl.org")) {
+        request <- paste0(request, "&redirect=no")
+    }
+    registry = bmRequest(request = request, verbose = verbose)
     
     ## check this looks like the MartRegistry XML, otherwise throw an error
     if(!grepl(x = registry, pattern = "^\n*<MartRegistry>")) {
@@ -121,7 +132,8 @@ listMarts <- function( mart = NULL, host="www.ensembl.org", path="/biomart/marts
             stop('Unexpected format to the list of available marts.\n',
                  'Please check the following URL manually, ',
                  'and try ?listMarts for advice.\n',
-                 request)
+                 request, 
+                 call. = FALSE)
         }
     }
     registry = xmlTreeParse(registry, asText=TRUE)
@@ -130,7 +142,7 @@ listMarts <- function( mart = NULL, host="www.ensembl.org", path="/biomart/marts
     marts = list(biomart = NULL, version = NULL, host = NULL, path = NULL, database = NULL)
     index = 1
     
-    if(host != "www.biomart.org" || archive){
+    # if(host != "www.biomart.org" || archive){
         for(i in seq(len=xmlSize(registry))){
             if(xmlName(registry[[i]])=="MartURLLocation"){  
                 if(xmlGetAttr(registry[[i]],"visible") == 1){
@@ -147,30 +159,13 @@ listMarts <- function( mart = NULL, host="www.ensembl.org", path="/biomart/marts
                 }
             }
         }
-    }
-    else{
-        for(i in seq(len=xmlSize(registry))){
-            if(xmlName(registry[[i]])=="MartURLLocation"){  
-                if(xmlGetAttr(registry[[i]],"visible") == 1){
-                    if(!is.null(xmlGetAttr(registry[[i]],"name"))) marts$biomart[index] = xmlGetAttr(registry[[i]],"name")
-                    if(!is.null(xmlGetAttr(registry[[i]],"database"))) marts$database[index] = xmlGetAttr(registry[[i]],"database")
-                    if(!is.null(xmlGetAttr(registry[[i]],"displayName"))) marts$version[index] = xmlGetAttr(registry[[i]],"displayName")
-                    marts$host[index] = host
-                    marts$path[index] = path
-                    marts$port[index] = 80
-                    if(!is.null(xmlGetAttr(registry[[i]],"serverVirtualSchema"))){
-                        marts$vschema[index] =  xmlGetAttr(registry[[i]],"serverVirtualSchema")
-                    }
-                    index=index+1
-                }
-            }
-        }
-    }
     if(includeHosts){
         return(marts)
     }
     else{
-        ret = data.frame(biomart = as.character(marts$biomart),version = as.character(marts$version), stringsAsFactors=FALSE)
+        ret = data.frame(biomart = as.character(marts$biomart),
+                         version = as.character(marts$version), 
+                         stringsAsFactors=FALSE)
         return(ret)
     } 
 }
@@ -181,14 +176,26 @@ listMarts <- function( mart = NULL, host="www.ensembl.org", path="/biomart/marts
 # #                           # #
 #################################
 
-useMart <- function(biomart, dataset, host = "www.ensembl.org", path = "/biomart/martservice", port = 80, archive = FALSE, ssl.verifypeer = TRUE, ensemblRedirect = NULL, version, verbose = FALSE){
+useMart <- function(biomart, dataset, host = "www.ensembl.org", path = "/biomart/martservice", port, 
+                     archive = FALSE, ensemblRedirect = NULL, version, verbose = FALSE) {
+    
+    if(missing(port)) {
+        port <- ifelse(grepl("https", host), yes = 443, no = 80)
+    }
     
     if(!is.null(ensemblRedirect)) {
         warning('The argument "ensemblRedirect" has been deprecated and will be removed in the next biomaRt release.')
-        ensemblRedirect <- NULL
     }
     
-    if(missing(biomart) && missing(version)) stop("No biomart databases specified. Specify a biomart database to use using the biomart or version argument")
+    mart <- .useMart(biomart, dataset, host = host, path = path, port = port, 
+                     archive = archive, version = version, verbose = verbose, ensemblRedirect = TRUE)
+}
+
+.useMart <- function(biomart, dataset, host = "www.ensembl.org", path = "/biomart/martservice", port = 80, 
+                    archive = FALSE, ensemblRedirect = NULL, version, verbose = FALSE){
+    
+    if(missing(biomart) && missing(version)) 
+        stop("No biomart databases specified. Specify a biomart database to use using the biomart or version argument")
     if(!missing(biomart)){ 
         if(!(is.character(biomart)))
             stop("biomart argument is not a string. ",
@@ -203,7 +210,7 @@ useMart <- function(biomart, dataset, host = "www.ensembl.org", path = "/biomart
     host <- .cleanHostURL(host)
     
     marts <- listMarts(host=host, path=path, port=port, includeHosts = TRUE,
-                       archive = archive, ssl.verifypeer = ssl.verifypeer)
+                       archive = archive)
     mindex = NA
     if(!missing(biomart)){ 
         mindex=match(biomart,marts$biomart)
@@ -217,23 +224,28 @@ useMart <- function(biomart, dataset, host = "www.ensembl.org", path = "/biomart
     if(is.na(mindex))
         stop("Incorrect BioMart name, use the listMarts function to see which BioMart databases are available")
     
-    if(is.na(marts$path[mindex]) || is.na(marts$vschema[mindex]) || is.na(marts$host[mindex]) || is.na(marts$port[mindex]) || is.na(marts$path[mindex])) stop("The selected biomart databases is not available due to error in the BioMart central registry, please report so the BioMart registry file can be fixed.")
+    if(is.na(marts$path[mindex]) || is.na(marts$vschema[mindex]) || 
+       is.na(marts$host[mindex]) || is.na(marts$port[mindex]) || 
+       is.na(marts$path[mindex])) 
+        stop("The selected biomart databases is not available due to error in the BioMart central registry, please report so the BioMart registry file can be fixed.")
+    
     if(marts$path[mindex]=="") marts$path[mindex]="/biomart/martservice" #temporary to catch bugs in registry
     #if(archive) biomart = marts$biomart[mindex]
     if(!missing(version)) biomart = marts$biomart[mindex]
     biomart = sub(" ","%20",biomart, fixed = TRUE, useBytes = TRUE)
     
     ## adding option to force use of specificed host with ensembl
-    #redirect <- ifelse(!ensemblRedirect && grepl(x = host, pattern = "ensembl.org"), 
-    #                   "?redirect=no",
-    #                   "")
+    redirect <- ifelse(!ensemblRedirect && grepl(x = host, pattern = "ensembl.org"), 
+                       "?redirect=no",
+                       "")
     
     mart <- new("Mart", 
                 biomart = biomart,
                 vschema = marts$vschema[mindex], 
                 host = paste0(host, ":", 
                               port,
-                              marts$path[mindex]))
+                              marts$path[mindex],
+                              redirect))
     
     if(length(grep("archive",martHost(mart)) > 0)){
         
@@ -244,12 +256,6 @@ useMart <- function(biomart, dataset, host = "www.ensembl.org", path = "/biomart
             mart@host <- stringr::str_replace(mart@host, pattern = current_release, "https://www.ensembl.org")
             mart@host <- stringr::str_replace(mart@host, pattern = ":80/", ":443/")
         }
-
-        #if(length(grep(reqHost,martHost(mart))) == 0){
-        #    message("Note: requested host was redirected from\n", reqHost, " to " , martHost(mart))
-        #    message("This often occurs when connecting to the archive URL for the current Ensembl release")
-        #    message("You can check the current version number using listEnsemblArchives()")
-        #}
     }
     
     BioMartVersion=bmVersion(mart, verbose=verbose)
@@ -497,8 +503,11 @@ filterType <- function(filter, mart){
 #getBM: generic BioMart query function   # 
 ##########################################
 
-getBM <- function(attributes, filters = "", values = "", mart, curl = NULL, checkFilters = TRUE, verbose=FALSE, uniqueRows=TRUE, bmHeader=FALSE, quote="\""){
+getBM <- function(attributes, filters = "", values = "", mart, curl = NULL, 
+                  checkFilters = TRUE, verbose=FALSE, uniqueRows=TRUE, bmHeader=FALSE, quote="\"",
+                  useCache = TRUE){
     
+    ## check the arguments are all valid
     martCheck(mart)
     if(missing( attributes ))
         stop("Argument 'attributes' must be specified.")
@@ -509,18 +518,35 @@ getBM <- function(attributes, filters = "", values = "", mart, curl = NULL, chec
         stop("Argument 'filters' must be a named list when sent as a list.")
     if(!is.list(filters) && all(filters != "") && missing( values ))
         stop("Argument 'values' must be specified.")
-    
     if(length(filters) > 0 && length(values) == 0)
         stop("Values argument contains no data.")
-    
     if(is.list(filters)){
         values = filters
         filters = names(filters)
     }
-    
     if(class(uniqueRows) != "logical")
         stop("Argument 'uniqueRows' must be a logical value, so either TRUE or FALSE")
     
+    ## determine if we should use the results cache
+    if(useCache) {
+        cache <- rappdirs::user_cache_dir(appname="biomaRt")
+        bfc <- BiocFileCache::BiocFileCache(cache, ask = FALSE)
+    }
+    hash <- .createHash(mart, attributes, filters, values, uniqueRows, bmHeader)
+    if( useCache && .checkCache(bfc, hash) ) {
+        
+        message("Cache found")
+        cache_hits <- bfcquery(bfc, hash, field = "rname")
+        if(nrow(cache_hits) > 1) {
+            stop("Multiple cache results found")
+        } else {
+            rid <- cache_hits$rid
+            load( bfc[[ rid ]] )
+            return(result)
+        }
+
+    } else { 
+    
     ## force the query to return the 'english text' header names with the result
     ## we use these later to match and order attribute/column names    
     callHeader <- TRUE
@@ -528,9 +554,9 @@ getBM <- function(attributes, filters = "", values = "", mart, curl = NULL, chec
                       martVSchema(mart),
                       "' uniqueRows = '",
                       as.numeric(uniqueRows),
-                      "' count = '0' datasetConfigVersion = '0.6' header='",
+                      "' count='0' datasetConfigVersion='0.6' header='",
                       as.numeric(callHeader),
-                      "' requestid= 'biomaRt'> <Dataset name = '",
+                      "' formatter='TSV' requestid='biomaRt'> <Dataset name = '",
                       martDataset(mart),"'>")
     
     #checking the Attributes
@@ -565,7 +591,7 @@ getBM <- function(attributes, filters = "", values = "", mart, curl = NULL, chec
     ## we submit a query for each chunk of the filter list
     for(i in seq_along(filterXmlList)) {
         
-        if(exists('pb')) {
+        if(i > 1) {
             pb$tick()
         }
         
@@ -579,45 +605,38 @@ getBM <- function(attributes, filters = "", values = "", mart, curl = NULL, chec
         ## we choose a separator based on whether '?redirect=no' is present
         sep <- ifelse(grepl(x = martHost(mart), pattern = ".+\\?.+"), "&", "?")
         
-        postRes <- .submitQueryXML(host = paste0(martHost(mart), sep),
+        ## create a unique name for this chunk & see if it has been run before
+        chunk_hash <- as(openssl::md5(paste(martHost(mart), fullXmlQuery)), "character")
+        tf <- file.path(tempdir(), paste0("biomaRt_", chunk_hash, ".rds"))
+        if(!file.exists(tf)) {
+            postRes <- .submitQueryXML(host = paste0(martHost(mart), sep),
                                    query = fullXmlQuery)
-        
-        if(verbose){
-            writeLines("#################\nResults from server:")
-            print(postRes)
-        }
-        if(!(is.character(postRes) && (length(postRes)==1L)))
-            stop("The query to the BioMart webservice returned an invalid result: biomaRt expected a character string of length 1. \nPlease report this on the support site at http://support.bioconductor.org")
-        
-        if(gsub("\n", "", postRes, fixed = TRUE, useBytes = TRUE) == "") { # meaning an empty result
-            
-            result = as.data.frame(matrix("", ncol=length(attributes), nrow=0), stringsAsFactors=FALSE)
-            
+            result <- .processResults(postRes, mart = mart, sep = sep, fullXmlQuery = fullXmlQuery,
+                                      verbose = verbose, callHeader = callHeader, 
+                                      quote = quote, attributes = attributes)
+            saveRDS(result, file = tf)
         } else {
-            
-            if(length(grep("^Query ERROR", postRes))>0L)
-                stop(postRes)
-            
-            ## convert the serialized table into a dataframe
-            con = textConnection(postRes)
-            result = read.table(con, sep="\t", header=callHeader, quote = quote, comment.char = "", check.names = FALSE, stringsAsFactors=FALSE)
-            if(verbose){
-                writeLines("#################\nParsed results:")
-                print(result)
-            }
-            close(con)
-            
-            if(!(is(result, "data.frame") && (ncol(result)==length(attributes)))) {
-                print(head(result))
-                stop("The query to the BioMart webservice returned an invalid result: the number of columns in the result table does not equal the number of attributes in the query. \nPlease report this on the support site at http://support.bioconductor.org")
-            }
+            result <- readRDS(tf)
         }
-        
         resultList[[i]] <- .setResultColNames(result, mart = mart, attributes = attributes, bmHeader = bmHeader)
     }
     ## collate results
     result <- do.call('rbind', resultList)
+    if(length(filterXmlList) > 1) {
+        pb$terminate()
+    }
+    
+    if(useCache) {
+        tf <- tempfile()
+        save(result, file = tf)
+        bfcadd(bfc, rname = hash, fpath = tf, action = "copy")
+        file.remove(tf)
+    }
+    
+    ## remove any temp chunk files
+    file.remove( list.files(tempdir(), pattern = "^biomaRt.*rds$", full.names = TRUE) )
     return(result)
+    }
 }
 
 ###################################
@@ -656,7 +675,7 @@ getLDS <- function(attributes, filters = "", values = "", mart, attributesL, fil
                        "\nPlease use the function 'listFilters' to get valid filter names"))
     }
     
-    xmlQuery = paste("<?xml version='1.0' encoding='UTF-8'?><!DOCTYPE Query><Query  virtualSchemaName = 'default' uniqueRows = '",as.numeric(uniqueRows),"' count = '0' datasetConfigVersion = '0.6' header='",as.numeric(bmHeader),"' requestid= \"biomaRt\"> <Dataset name = '",martDataset(mart),"'>",sep="")
+    xmlQuery = paste("<?xml version='1.0' encoding='UTF-8'?><!DOCTYPE Query><Query  virtualSchemaName = 'default' uniqueRows = '",as.numeric(uniqueRows),"' count = '0' datasetConfigVersion = '0.6' header='",as.numeric(bmHeader),"' formatter = 'TSV' requestid= 'biomaRt'> <Dataset name = '",martDataset(mart),"'>",sep="")
     attributeXML = paste("<Attribute name = '", attributes, "'/>", collapse="", sep="")
     if(length(filters) > 1){
         if(class(values)!= "list")
@@ -763,10 +782,10 @@ getLDS <- function(attributes, filters = "", values = "", mart, attributesL, fil
     postRes <- .submitQueryXML(host = paste0(martHost(mart), sep),
                                query = xmlQuery)
     
-    ## 10-01-2014
+
     if(length(grep("^Query ERROR", postRes))>0L)
         stop(postRes)  
-    ##
+
     if(postRes != ""){
         con = textConnection(postRes)
         result = read.table(con, sep="\t", header=bmHeader, quote = "\"", comment.char = "", as.is=TRUE, check.names = TRUE)
@@ -774,15 +793,17 @@ getLDS <- function(attributes, filters = "", values = "", mart, attributesL, fil
         
         if(nrow(result) > 0 && all(is.na(result[,ncol(result)])))
             result = result[,-ncol(result),drop=FALSE]
-        ## 10 - 01 - 2014
+
         res_attributes <- c(attributes,attributesL)
         if(!(is(result, "data.frame") && (ncol(result)==length(res_attributes)))) {
             print(head(result))
-            stop("The query to the BioMart webservice returned an invalid result: the number of columns in the result table does not equal the number of attributes in the query. \nPlease report this on the support site at http://support.bioconductor.org")
+            stop("The query to the BioMart webservice returned an invalid result: ", 
+            "the number of columns in the result table does not equal the number of attributes in the query. \n",
+            "Please report this on the support site at http://support.bioconductor.org")
         } 
         if(!bmHeader){  #assumes order of results same as order of attibutes in input
             colnames(result) = res_attributes
-            ##
+
         }
     } else {
         warning("getLDS returns NULL.")
@@ -797,17 +818,12 @@ getLDS <- function(attributes, filters = "", values = "", mart, attributesL, fil
 
 getXML <- function(host="http://www.ensembl.org/biomart/martservice?", xmlquery){
     
-    ## Deprecated 29-01-2018
-    .Deprecated("biomaRt:::.submitQueryXML", 
-                msg = paste0("Function 'getXML()' is deprecated.\n",
+    ## Deprecated   29-01-2018
+    ## Defunct      03-06-2019
+    .Defunct("biomaRt:::.submitQueryXML", 
+                msg = paste0("Function 'getXML()' is defunct\n",
                              "Use 'biomaRt:::.submitQueryXML' instead\n",
                              "See help('getXML') for further details"))
-    
-    pf = postForm(host,"query"=xmlquery)
-    con = textConnection(pf)
-    result = read.table(con, sep="\t", header=FALSE, quote = "", comment.char = "", as.is=TRUE)
-    close(con)
-    return(result)
 }
 
 ######################
