@@ -124,4 +124,23 @@ test_that("URL formatting works", {
     host <- 'ensembl.org'
     expect_equal(object = .cleanHostURL(host = host),
                  expected = "http://www.ensembl.org")
+})
+
+#############################
+context("Query submission")
+#############################
+
+test_that("", {
+    host <- "https://www.ensembl.org:443/biomart/martservice?"
+    query <- "<?xml version='1.0' encoding='UTF-8'?><!DOCTYPE Query>
+            <Query virtualSchemaName='default' uniqueRows='1' count='0' datasetConfigVersion='0.6' header='1' requestid='biomaRt' formatter='TSV'>
+            <Dataset name = 'hsapiens_gene_ensembl'><Attribute name = 'ensembl_gene_id'/>
+            <Attribute name = 'hgnc_symbol'/>
+            <Filter name = \"ensembl_gene_id\" value = \"ENSG00000100036\" />
+            </Dataset></Query>"
+    expect_silent(res_tsv <- biomaRt:::.submitQueryXML(host, query))
+    expect_silent(res_html <- biomaRt:::.fetchHTMLresults(host, query))
+    expect_identical(res_html,
+                     read.table(textConnection(res_tsv), sep = "\t", header = TRUE, 
+                                check.names = FALSE, stringsAsFactors = FALSE))
 })
\ No newline at end of file
